﻿---------------------------------------------------------------------------------
--
-- Prat - A framework for World of Warcraft chat mods
--
-- Copyright (C) 2006-2007  Prat Development Team
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
-------------------------------------------------------------------------------

--[[
Name: Prat_CustomFilters
Revision: $Revision: 33646 $
Author(s):  Sylvanaar (sylvanaar@mindspring.com)
Website: http://files.wowace.com/Prat/
Documentation: http://www.wowace.com/wiki/Prat/Integrated_Modules#CustomFilters
Subversion: http://svn.wowace.com/wowace/trunk/Prat/
Discussions: http://groups.google.com/group/wow-prat
Issues and feature requests: http://code.google.com/p/prat/issues/list
Description: Module to support custom filters. (default=off).
Dependencies: Prat
]]

-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("CustomFilters")

if PRAT_MODULE == nil then 
    return 
end

-- define localized strings
local L = AceLibrary("AceLocale-2.2"):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
	["CustomFilters"] = true,
	["Module to support custom filters."] = true,
    ["Add Pattern"] = true,
    ["Add a pattern to search for."] = true,
    ["Remove Pattern"] = true,
    ["Remove an existing pattern."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
    ["CustomFilters"] = "사용자 필터",
    ["Module to support custom filters."] = "사용자 필터를 지원하는 모듈입니다.",
    ["Add Pattern"] = "패턴 추가",
    ["Add a pattern to search for."] = "탐색할 패턴을 추가합니다.",
    ["Remove Pattern"] = "패턴 제거",
    ["Remove an existing pattern."] = "기존의 패턴을 삭제합니다.",
} end)

local cat = Prat.Categories

Prat_CustomFilters = Prat:NewModule(PRAT_MODULE)
Prat_CustomFilters.revision = tonumber(string.sub("$Revision: 33646 $", 12, -3))

Prat_CustomFilters.moduleName	= L["CustomFilters"] 	-- module name, eg "ExampleMod"
Prat_CustomFilters.moduleDesc	= L["Module to support custom filters."] 	-- module description, eg "An example of a Prat 2 mod"

Prat_CustomFilters.consoleName	= string.lower(Prat_CustomFilters.moduleName)
Prat_CustomFilters.guiName		= Prat_CustomFilters.moduleName

Prat_CustomFilters.Categories	= { cat.FILTER, cat.FORMATTING, cat.TEXT, cat.COLORS }

Prat_CustomFilters.moduleOptions = {}

Prat_CustomFilters.defaultDB = {
	on	= false, -- this value tracks if the module should be on or off, 
	             -- an option 'toggle' is automatically created so you can 
	             -- set this value
	inbound = {
	},
	outbound = {
	}
}


Prat_CustomFilters.toggleOptions = {
}

function Prat_CustomFilters:GetModuleOptions()
	self.moduleOptions = {
		name	= self.moduleName,
		desc	= self.moduleDesc,
		type	= "group",
		args = {
		    inbound = {
		        type = "group",
		        name = "Inbound",
		        desc = "Inbound",
		        args = {
		        }
		    },
		    outbound = {
		        type = "group",
		        name = "Outbound",
		        desc = "Outbound",
		        args = {
		        }
		    }		    		    
		}
	}
    
    local modeOpts = self.moduleOptions.args
    for k,v in pairs(modeOpts) do 
        self:BuildModeOptions(k, modeOpts)
    end

	return self.moduleOptions
end

function Prat_CustomFilters:BuildModeOptions(mode, opts)
    local mode = mode  
    local po = opts[mode].args

    self[mode] = {}
    self[mode].validate = {}
    
    po.pathdr = {
        type = "header",
        name = "Pattern Options",
        order = 80,
    }

    for k,v in pairs(self.db.profile[mode]) do 
        self:AddPatternOptions(po, k, mode)
    	table.insert(self[mode].validate, k)      
    end

    po.opspc = {
        type = "header",
        order = 94,
    }

    po.addpattern = {
        name = L["Add Pattern"],
        desc = L["Add a pattern to search for."],
        type = "text",
        usage = "<string>",
        get = false,
        passValue = mode,
        set = "AddPattern"
    }

    po.removepattern = {
        name = L["Remove Pattern"],
        desc = L["Remove an existing pattern."],
        type = "text",
    	current = "",
        passValue = mode,
    	get = function() return "" end,
    	set = "RemovePattern",
    	validate = self[mode].validate,
        disabled = function(mode) 
            for k,v in pairs(self.db.profile[mode]) do 
                return false             
            end return true end,
    }
    
end
 
function Prat_CustomFilters:AddPatternOptions(o, pattern, mode)
    o[pattern] = o[pattern] or {}
    local po = o[pattern]

    local mode = mode
    local pattern = pattern
    local passval = self.db.profile[mode][pattern]     
    po.type = "group"
    po.name = pattern
    po.desc = pattern
    po.order = 90
    po.passValue = passval
    po.args = {
        searchfor = {
            type = "text",
            name = "Search Pattern",
            desc = "Search Pattern",
            usage = "<string>",
            passValue = passval,
            get = "GetPatternSearch",
            set = "SetPatternSearch"
        },
        replacewith = {
            type = "text",
            name = "Replacement Text",
            desc = "Replacement Text",
            usage = "<string>",
            passValue = passval,
            get = "GetPatternReplace",
            set = "SetPatternReplace"
        },
        hilight = {
            type = "toggle",
            passValue = passval,
            name = "Hilight Match Text",
            desc = "Hilight Match Text",
            get = "GetPatternHilight",
            set = "SetPatternHilight"
        },
        hilight_color = {
            type = "color",
            passValue = passval,
            name = "Hilight Color",
            desc = "Hilight Color",
            get = "GetPatternHilightClr",
            set = "SetPatternHilightClr"
        },
}       
end


local function match(text, matchopts, mode)
    Prat_CustomFilters:Debug("match", text, mode)
    local matchtype
    if mode == "inbound" then 
        matchtype = "FRAME"
    else
        matchtype = "OUTBOUND"
    end
    
    local textout = textout
    
    if matchopts.replacewith then 
        textout = matchopts.replacewith
    end
    
    if matchopts.hilight then 
        local hexcolor = CLR:GetHexColor(matchopts.hilight_color)
        textout = CLR:Colorize(hexcolor, textout)
    end 

    Prat_CustomFilters:Debug("matchresult", textout)
    
    return Prat:RegisterMatch(textout, matchtype)    
end

Prat_CustomFilters.modulePatterns = {}

function Prat_CustomFilters:RegisterPattern(matchopts, mode)
    local mode = mode
    local matchopts = matchopts
    if mode == "inbound" then 
        matchtype = "FRAME"
    else
        matchtype = "OUTBOUND"
    end    
    local patterninfo = { pattern = matchopts.searchfor,
                      matchfunc = 
                        function(text, ...)
                            return match(text, matchopts, mode)
                        end,
                      type = matchtype }                                             

    local idx = Prat:RegisterPattern(patterninfo, self)
    
    patterninfo.idx = idx
    
    table.insert(self.modulePatterns, patterninfo)
end

--[[------------------------------------------------
    Module Event Functions
------------------------------------------------]]--

-- things to do when the module is enabled
function Prat_CustomFilters:OnModuleEnable()
    local modeOpts = self.moduleOptions.args
    local mode
    for mode,_ in pairs(modeOpts) do 
        if type(self.db.profile[mode]) == "table" then 
            local patopts
            for _,patopts in pairs(self.db.profile[mode]) do
                self:RegisterPattern(patopts, mode) 
            end
        end 
    end
end

-- things to do when the module is disabled
function Prat_CustomFilters:OnModuleDisable()
end


--[[------------------------------------------------
    Core Functions
------------------------------------------------]]--
function Prat_CustomFilters:GetPatternSearch(p)
    self:Debug(p)
    return p.searchfor
end
function Prat_CustomFilters:SetPatternSearch(p, v)
    self:Debug(p, v)
    p.searchfor = v
end
function Prat_CustomFilters:GetPatternReplace(p)
    return p.replacewith
end
function Prat_CustomFilters:SetPatternReplace(p, v)
    p.replacewith = v
end
function Prat_CustomFilters:GetPatternHilight(p)
    return p.hilight
end
function Prat_CustomFilters:SetPatternHilight(p, v)
    p.hilight = v
end

local white_clr = {r=1.0, b=1.0, g=1.0}
function Prat_CustomFilters:GetPatternHilightClr(p)
    local h = p.hilight_color or white_clr
    return h.r or 1.0, h.g or 1.0, h.b or 1.0
end
function Prat_CustomFilters:SetPatternHilightClr(p, r,g,b)
    p.hilight_color = p.hilight_color or {}
    local h = p.hilight_color
    h.r, h.g, h.b = r, g, b
end



function Prat_CustomFilters:AddPattern(mode, pattern)
    self:Debug("Add "..mode..": "..pattern)
    local p = self.db.profile[mode]
    
	for k, v in pairs(p) do
		if v.name == pattern then
			return
		end
	end

    p[pattern] = {}
	
	self[mode].validate = self[mode].validate or {}
	local v = self[mode].validate
	table.insert(v, pattern)
	
	local o = self.moduleOptions.args[mode].args
    self:AddPatternOptions(o, pattern, mode)
end

function Prat_CustomFilters:RemovePattern(mode, pattern)
    self:Debug("Remove "..mode..": "..pattern)
    local p = self.db.profile[mode]

    p[pattern] = nil    
	self[mode].validate[pattern] = nil
	
	self.moduleOptions.args[mode].args[pattern] = nil
end    



